#ifndef USING_ENGINUITY
#define USING_ENGINUITY
#endif
#include "engine.h"
#include "interpolators.h"

std::list< CMMPointer<IInterpolator> > IInterpolator::interpolators;

IInterpolator::IInterpolator(float &t) : target(t)
{
	interpolators.push_back(this);
	bFreeze=false;
}

IInterpolator::~IInterpolator()
{

}

void IInterpolator::Kill()
{
	std::list< CMMPointer<IInterpolator> >::iterator it=(std::find(interpolators.begin(),interpolators.end(),this));
	if(it!=interpolators.end())
		(*it)=0;
}

bool CInterpolatorUpdater::Start() { return true; }
void CInterpolatorUpdater::Stop() { IInterpolator::interpolators.clear(); }
void CInterpolatorUpdater::Update()
{
	PROFILE("Interpolator task");
	std::list< CMMPointer<IInterpolator> >::iterator it, ite=IInterpolator::interpolators.end(), itT;
	for(it=IInterpolator::interpolators.begin(); it!=ite; it++)
	{
		if((*it).isValid())
		{
			(*it)->Update(CGlobalTimer::dT);
		}else{
			itT=it;
			--it;
			IInterpolator::interpolators.erase(itT);
		}
	}
}

ITimebasedInterpolator::ITimebasedInterpolator(float &targ, float time) : IInterpolator(targ)
{
	elapsedTime=0;
	totalTime=time;
}

void ITimebasedInterpolator::Update(float dt)
{
	if(bFreeze)return;
	elapsedTime+=dt;
	Calculate();
	if(elapsedTime>totalTime)
	{
		Kill();
	}
}

void ITimebasedInterpolator::Reset()
{
	elapsedTime=0;
}

CLinearTimeInterpolator::CLinearTimeInterpolator(float &targ, float time, float sV, float eV) : ITimebasedInterpolator(targ, time)
{
	startVal=sV; endVal=eV;
	Calculate();
}

void CLinearTimeInterpolator::Calculate()
{
	float b=clamp(elapsedTime/totalTime,0,1);
	target = startVal*(1-b) + endVal*b;
}

CQuadraticTimeInterpolator::CQuadraticTimeInterpolator(float &targ, float time, float sV, float mV, float eV) : ITimebasedInterpolator(targ, time)
{
	startVal=sV; midVal=mV; endVal=eV;
}

void CQuadraticTimeInterpolator::Calculate()
{
	float b=clamp(elapsedTime/totalTime,0,1), a=1-b;
	target = startVal*a*a + midVal*2*a*b + endVal*b*b;
}

CCubicTimeInterpolator::CCubicTimeInterpolator(float &targ, float time, float sV, float mV1, float mV2, float eV) : ITimebasedInterpolator(targ, time)
{
	startVal=sV; midVal1=mV1; midVal2=mV2; endVal=eV;
}

void CCubicTimeInterpolator::Calculate()
{
	float b=clamp(elapsedTime/totalTime,0,1), a=1-b;
	target = startVal*a*a*a + midVal1*3*a*a*b + midVal2*3*a*b*b + endVal*b*b*b;
}