#ifndef INTERPOLATORS_H_INCLUDED
#define INTERPOLATORS_H_INCLUDED

class CInterpolatorUpdater : public ITask
{
public:
	bool Start();
	void Update();
	void Stop();
	AUTO_SIZE;
};


class IInterpolator : public IMMObject
{
protected:
	float &target;
	static std::list< CMMPointer<IInterpolator> > interpolators;
public:
	IInterpolator(float &t);
	virtual ~IInterpolator();

	bool bFreeze;

	virtual void Update(float dt)=0;
	void Kill();

	friend class CInterpolatorUpdater;
	AUTO_SIZE;
};


class ITimebasedInterpolator : public IInterpolator
{
protected:
	float elapsedTime, totalTime;
	virtual void Calculate()=0;
public:
	void Reset();
	void Update(float dt);	
	ITimebasedInterpolator(float &targ, float time);
	AUTO_SIZE;
};

class CLinearTimeInterpolator : public ITimebasedInterpolator
{
protected:
	float startVal, endVal;
	void Calculate();
public:
	CLinearTimeInterpolator(float &targ, float time, float sV, float eV);
	AUTO_SIZE;
};

class CQuadraticTimeInterpolator : public ITimebasedInterpolator
{
protected:
	float startVal, midVal, endVal;
	void Calculate();
public:
	CQuadraticTimeInterpolator(float &targ, float time, float sV, float mV, float eV);
	AUTO_SIZE;
};

class CCubicTimeInterpolator : public ITimebasedInterpolator
{
protected:
	float startVal, midVal1, midVal2, endVal;
	void Calculate();
public:
	CCubicTimeInterpolator(float &targ, float time, float sV, float mV1, float mV2, float eV);
	AUTO_SIZE;
};

#endif