#ifndef MATH_H_INCLUDED
#define MATH_H_INCLUDED

const float PI=3.141592f;

#include <list>
#include <cmath>

class matrix;
class vec3d;
class quaternion;

class matrix
{
	class matrixRow
	{
	public:
		float *dta;
		inline float &operator [](int col){return dta[col];}
	};
	matrixRow rows[4];
public:
	matrix(){rows[0].dta=data[0]; rows[1].dta=data[1]; rows[2].dta=data[2];rows[3].dta=data[3];
	data[0][0]=data[1][1]=data[2][2]=data[3][3]=1.0f;
	data[1][0]=data[2][0]=data[3][0]=0.0f;
	data[0][1]=data[2][1]=data[3][1]=0.0f;
	data[0][2]=data[1][2]=data[3][2]=0.0f;
	data[0][3]=data[1][3]=data[2][3]=0.0f;}

	float data[4][4];
	matrixRow &operator [](int row){return rows[row];}
	operator float**(){return (float**)data;}

	void translate(float x, float y, float z)
	{
		data[3][0]=x;
		data[3][1]=y;
		data[3][2]=z;
	}
	void rotateX(float radians)
	{
		data[1][1]=(float)cos(radians);
		data[2][1]=(float)sin(radians);
		data[1][2]=(float)-sin(radians);
		data[2][2]=(float)cos(radians);
	}
	void rotateY(float radians)
	{
		data[0][0]=(float)cos(radians);
		data[2][0]=(float)-sin(radians);
		data[0][2]=(float)sin(radians);
		data[2][2]=(float)cos(radians);
	}
	void rotateZ(float radians)
	{
		data[0][0]=(float)cos(radians);
		data[1][0]=(float)sin(radians);
		data[0][1]=(float)-sin(radians);
		data[1][1]=(float)cos(radians);
	}
	void followedBy(matrix &m)
	{
		matrix res;
		for(int i=0;i<3;i++)
		{
			for(int j=0;j<3;j++)
			{
				res[i][j]=data[i][0]*m[0][j] + data[i][1]*m[1][j]+data[i][2]*m[2][j];
				//res[i][j]=data[i][0]*data[0][j] + m[i][1]*data[1][j]+m[i][2]*data[2][j];
			}
		}
		memcpy(data,res.data,sizeof(float)*4*4);
	}
	
	void getYPR(float &y, float &p, float &r)
	{
		y=(float)atan(data[1][0]/data[0][0]);
		p=(float)asin(-data[2][0]);
		r=(float)atan(data[2][1]/data[2][2]);
	}
};

class vec2d
{
public:
	float x,y;
	vec2d(){x=y=0;}
	vec2d(float cx,float cy){x=cx;y=cy;}
	vec2d(const vec2d &s){x=s.x;y=s.y;}

	vec2d operator +(vec2d &v){return vec2d(x+v.x,y+v.y);}
	vec2d operator -(vec2d &v){return vec2d(x-v.x,y-v.y);}
	vec2d operator +(float f){return vec2d(x+f, y+f);}
	vec2d operator -(float f){return vec2d(x-f, y-f);}
	vec2d operator *(float f){return vec2d(x*f, y*f);}
	vec2d operator /(float f){return vec2d(x/f, y/f);}

	vec2d operator -(){return vec2d(-x,-y);}

	vec2d &operator +=(vec2d &v){x+=v.x;y+=v.y;return (*this);}
	vec2d &operator -=(vec2d &v){x-=v.x;y-=v.y;return (*this);}
	vec2d &operator +=(float f){x+=f;y+=f;return (*this);}
	vec2d &operator -=(float f){x-=f;y-=f;return (*this);}
	vec2d &operator *=(float f){x*=f;y*=f;return (*this);}
	vec2d &operator /=(float f){x/=f;y/=f;return (*this);}

	float dot(vec2d &v){return x*v.x+y*v.y;}
	float magsq(){return x*x+y*y;}
	float magnitude(){return (float)sqrt(magsq());}
	vec2d &normalize(){*this/=magnitude(); return *this;}
	vec2d apply(matrix &v)
	{
		vec2d res;
		res.x= x*v[0][0] + y*v[1][0] + v[3][0];
		res.y= x*v[0][1] + y*v[1][1] + v[3][1];
		return res;
	}

	bool operator == (vec2d &v) {if((x==v.x)&&(y==v.y))return true; return false;}
	bool operator != (vec2d &v) {return !(*this==v);}

};

class vec3d
{
public:
	float x,y,z;
	vec3d(){x=y=z=0;}
	vec3d(float cx,float cy, float cz){x=cx;y=cy;z=cz;}
	vec3d(const vec3d &s){x=s.x;y=s.y;z=s.z;}

	vec3d operator +(vec3d &v){return vec3d(x+v.x,y+v.y,z+v.z);}
	vec3d operator -(vec3d &v){return vec3d(x-v.x,y-v.y,z-v.z);}
	vec3d operator +(float f){return vec3d(x+f, y+f, z+f);}
	vec3d operator -(float f){return vec3d(x-f, y-f, z-f);}
	vec3d operator *(float f){return vec3d(x*f, y*f, z*f);}
	vec3d operator /(float f){return vec3d(x/f, y/f, z/f);}

	vec3d operator -(){return vec3d(-x,-y,-z);}

	vec3d &operator +=(vec3d &v){x+=v.x;y+=v.y;z+=v.z;return (*this);}
	vec3d &operator -=(vec3d &v){x-=v.x;y-=v.y;z-=v.z;return (*this);}
	vec3d &operator +=(float f){x+=f;y+=f;z+=f;return (*this);}
	vec3d &operator -=(float f){x-=f;y-=f;z-=f;return (*this);}
	vec3d &operator *=(float f){x*=f;y*=f;z*=f;return (*this);}
	vec3d &operator /=(float f){x/=f;y/=f;z/=f;return (*this);}

	float dot(vec3d &v){return x*v.x+y*v.y+z*v.z;}
	float magsq(){return x*x+y*y+z*z;}
	float magnitude(){return (float)sqrt(magsq());}
	vec3d cross(vec3d &v){return vec3d(y*v.z-z*v.y,z*v.x-x*v.z,x*v.y-y*v.x);}
	vec3d &normalize(){*this/=magnitude(); return *this;}
	vec3d apply(matrix &v)
	{
		vec3d res;
		res.x= x*v[0][0] + y*v[1][0] + z*v[2][0]+v[3][0];
		res.y= x*v[0][1] + y*v[1][1] + z*v[2][1]+v[3][1];
		res.z= x*v[0][2] + y*v[1][2] + z*v[2][2]+v[3][2];
		return res;
	}

	bool operator == (vec3d &v) {if((x==v.x)&&(y==v.y)&&(z==v.z))return true; return false;}
	bool operator != (vec3d &v) {return !(*this==v);}

};

class quaternion
{
public:
	vec3d v; float w;
	quaternion(){w=0.0f;}
	quaternion(float a, float b, float c, float d){v.x=a;v.y=b;v.z=c;w=d;}
	quaternion(vec3d &a, float b){v=a;w=b;}
	quaternion(quaternion &q){v=q.v;w=q.w;}
	quaternion(float y, float p, float r)
	{
		double cyaw = cos(y*0.5f), cpitch = cos(p*0.5f), croll = cos(r*0.5f);
		double syaw = sin(y*0.5f), spitch = sin(p*0.5f), sroll = sin(r*0.5f);

		v.x = (float)(cyaw*cpitch*sroll - syaw*spitch*croll);
		v.y = (float)(cyaw*spitch*croll + syaw*cpitch*sroll);
		v.z = (float)(syaw*cpitch*croll - cyaw*spitch*sroll);
		w   = (float)(cyaw*cpitch*croll + syaw*spitch*sroll);
	}

	quaternion &operator =(quaternion &q) { v=q.v; w=q.w; return *this; }

	float magsq(){return v.magsq()+w*w;}
	float magnitude(){return (float)sqrt(magsq());}

	quaternion &operator +=(quaternion &q) { w+=q.w; v+=q.v; return *this; }
	quaternion operator +(quaternion &q) { return quaternion(v+q.v,w+q.w); }
	quaternion &operator -=(quaternion &q) { w-=q.w; v-=q.v; return *this; }
	quaternion operator -(quaternion &q) { return quaternion(v-q.v,w-q.w); }
	quaternion &operator *=(float s) { w*=s; v*=s; return *this; }
	quaternion operator *(float s) { return quaternion(v*s,w*s); }
	quaternion &operator /=(float s) { w/=s; v/=s; return *this; }
	quaternion operator /(float s) { return quaternion(v/s,w/s); }
	quaternion operator ~(){ return quaternion(-v,w); }

	quaternion operator *(quaternion &q) { return quaternion(q.v*w + v*q.w + v.cross(q.v), w*q.w - v.dot(q.v)); }
	quaternion operator *=(quaternion &q) { (*this)=(*this)*q; return *this;}

	quaternion operator *(vec3d &v) { return (*this)*quaternion(v,0); }
	quaternion operator *=(vec3d &v) { (*this)=(*this)*v; return *this; }

	float angle() { return (float)(2*acos(w)); }
	vec3d axis() { return vec3d(v).normalize(); }

	quaternion &rotate(quaternion &q) { (*this)=(*this)*q*(~(*this)); return (*this); }
	vec3d rotate(vec3d &v) { return (((*this)*v)*(~(*this))).v; }
	
	matrix toMatrix()
	{
		matrix m;

		m[0][0]=  w*w-v.magsq()+2*v.x*v.x;
		m[0][1]=2*v.x*v.y - 2*v.z*w;
		m[0][2]=2*v.x*v.z + 2*v.y*w;

		m[1][0]=2*v.x*v.y + 2*v.z*w;
		m[1][1]=w*w-v.magsq()+2*v.y*v.y;
		m[1][2]=2*v.y*v.z -2*v.x*w;

		m[2][0]=2*v.z*v.x - 2*v.y*w;
		m[2][1]=2*v.z*v.y + 2*v.x*w;
		m[2][2]=w*w-v.magsq()+2*v.z*v.z;

		return m;
	}

	void getYPR(float &y, float &p, float &r)
	{
		matrix &m=toMatrix();
		m.getYPR(y,p,r);
	}
};

struct axis
{
	vec3d origin;
	vec3d x,y,z;
	axis() { defaultInit(); }
	axis(quaternion &q){defaultInit(); x=q.rotate(x); y=q.rotate(y); z=q.rotate(z); normalize();}
	axis(vec3d &o){defaultInit(); origin=o;}
	axis(vec3d &o, quaternion &q)
	{
		defaultInit();
		x=q.rotate(x);
		y=q.rotate(y);
		z=q.rotate(z);
		origin=o;
		normalize();
	}
	axis(const axis &a){x=a.x;y=a.y;z=a.z;origin=a.origin;}
	
	void normalize(){x.normalize(); y.normalize(); z.normalize();}

	axis &operator =(axis &a){x=a.x;y=a.y;z=a.z;origin=a.origin;return *this;}

	vec3d toLocal(vec3d &input)
	{
		vec3d res;
		res=vec3d((input-origin).dot(x),(input-origin).dot(y),(input-origin).dot(z));
		return res;
	}

	vec3d toGlobal(vec3d &input)
	{
		vec3d res;
		res=x*input.x+y*input.y+z*input.z;
		res+=origin;
		return res;
	}
	private:
	void inline defaultInit(){x=vec3d(1,0,0); y=vec3d(0,1,0); z=vec3d(0,0,1);}
};

class plane
{
public:
	float a,b,c,d;
	plane(){a=b=c=d=0.0f;}
	bool isFront(vec3d &pt)
	{
		return ((pt.x*a)+(pt.y*b)+(pt.z*c)>=d);
	}
	bool isBack(vec3d &pt){return !isFront(pt);}
	void normalize(){ float m=vec3d(a,b,c).magnitude(); a/=m; b/=m; c/=m; d/=m;}
};

float inline toRadians(float val)
{
	return val*(PI/180);
}

float inline toDegrees(float val)
{
	return val/(PI/180);
}

#endif